'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TOKEN = exports.SETTINGS = exports.LOGGER = exports.SERVER = undefined;
exports.buildGQLClient = buildGQLClient;
exports.login = login;
exports.fetchCurrentUser = fetchCurrentUser;
exports.createAccount = createAccount;
exports.updateAccount = updateAccount;
exports.addUserToAccount = addUserToAccount;
exports.updatePassword = updatePassword;
exports.createProduct = createProduct;
exports.updateProduct = updateProduct;
exports.createProperty = createProperty;
exports.updateProperty = updateProperty;

var _apolloClient = require('apollo-client');

var _apolloClient2 = _interopRequireDefault(_apolloClient);

var _preferences = require('preferences');

var _preferences2 = _interopRequireDefault(_preferences);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _cliLogger = require('cli-logger');

var _cliLogger2 = _interopRequireDefault(_cliLogger);

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-next-line no-unused-vars
const SERVER = exports.SERVER = process.env.CRYPTID_SERVER || 'https://cryptid.adorable.io';
const LOGGER = exports.LOGGER = (0, _cliLogger2.default)({ level: _cliLogger2.default.INFO });

function buildUrl(uri) {
  return _url2.default.resolve(SERVER, uri);
}

function loadSettings() {
  let name = `com.adorable.cryptid.${(0, _md2.default)(SERVER)}`;
  let settings = new _preferences2.default(name, { server: SERVER, token: '', email: '' });

  let loggedIn = settings.token.length > 0 && settings.email.length > 0;
  settings.loggedIn = loggedIn;
  settings.checkLogin = () => {
    if (!loggedIn) {
      LOGGER.info(_chalk2.default.red('You must first login with "cryptid login"'));
      process.exit(1);
    }
  };
  settings.clear = function () {
    this.token = '';
    this.email = '';
    this.loggedIn = false;
    delete this.checkLogin;
  };
  settings.setData = function (token, email) {
    this.token = token;
    this.email = email;
  };

  return settings;
}
const SETTINGS = exports.SETTINGS = loadSettings();
const TOKEN = exports.TOKEN = SETTINGS.token;

function checkError(error) {
  if (error) {
    switch (error.code) {
      case 'ENOTFOUND':
        LOGGER.info(_chalk2.default.red(`Could not reach cryptid server. Is ${SERVER} reachable?`));
        break;
      case 'ECONNREFUSED':
        LOGGER.info(_chalk2.default.red(`Could not reach cryptid server. Is ${SERVER} reachable?`));
        break;
      default:
        LOGGER.error(_chalk2.default.red(`An error occurred communicating with ${SERVER}`));
        break;
    }
    process.exit(1);
  }
}

/* =========================================================
 *
 *  GraphQL Functions
 *
 *  ========================================================= */

function buildGQLClient() {
  return new _apolloClient2.default({
    networkInterface: (0, _apolloClient.createNetworkInterface)({
      uri: _url2.default.resolve(SERVER, '/api/reports'),
      opts: {
        headers: { Authorization: `Token token=${TOKEN}` }
      }
    })
  });
}

/* =========================================================
 *
 *  Cryptid Service Functions
 *
 * ========================================================= */

function login(username, password) {
  let options = {
    url: buildUrl('/api/sessions'),
    method: 'post',
    json: {
      user: {
        email: username,
        password: password
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {

    if (response.statusCode === 201) {
      SETTINGS.setData(body.data.token, username);
      process.exit(0);
    } else {
      LOGGER.info(_chalk2.default.red('Invalid password'));
      process.exit(1);
    }
  });
}

function fetchCurrentUser(callback) {
  let options = {
    url: buildUrl('/api/users/current'),
    method: 'get',
    json: true,
    headers: { Authorization: `Token token=${TOKEN}` }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function createAccount(accountName, callback) {
  let options = {
    url: buildUrl('/api/accounts'),
    method: 'post',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      account: {
        name: accountName
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function updateAccount(accountId, accountName, callback) {
  let options = {
    url: buildUrl(`/api/accounts/${accountId}`),
    method: 'put',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      account: {
        name: accountName
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function addUserToAccount(accountId, email, callback) {
  let options = {
    url: buildUrl(`/api/accounts/${accountId}/users`),
    method: 'post',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      user: {
        email: email
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function updatePassword(passwords, callback) {
  let currentPassword = passwords.currentPassword,
      newPassword = passwords.newPassword,
      newPasswordConfirm = passwords.newPasswordConfirm;


  let options = {
    url: buildUrl('/api/users/current'),
    method: 'put',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      user: {
        currentPassword: currentPassword,
        password: newPassword,
        passwordConfirmation: newPasswordConfirm
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function createProduct(accountId, productName, callback) {
  let options = {
    url: buildUrl(`/api/accounts/${accountId}/products`),
    method: 'post',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      product: {
        name: productName
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function updateProduct(data, callback) {
  let accountId = data.accountId,
      productId = data.productId,
      productName = data.productName;


  let options = {
    url: buildUrl(`/api/accounts/${accountId}/products/${productId}`),
    method: 'put',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      product: {
        name: productName
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function createProperty(data, callback) {
  let accountId = data.accountId,
      productId = data.productId,
      propertyName = data.propertyName,
      propertyType = data.propertyType;


  let options = {
    url: buildUrl(`/api/accounts/${accountId}/products/${productId}/properties`),
    method: 'post',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      property: {
        name: propertyName,
        propertyType: propertyType
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}

function updateProperty(data, callback) {
  let accountId = data.accountId,
      productId = data.productId,
      propertyName = data.propertyName,
      propertyId = data.propertyId;


  let options = {
    url: buildUrl(`/api/accounts/${accountId}/products/${productId}/properties/${propertyId}`),
    method: 'put',
    headers: { Authorization: `Token token=${TOKEN}` },
    json: {
      property: {
        name: propertyName
      }
    }
  };

  (0, _request2.default)(options, (error, response, body) => {
    checkError(error);
    callback(response, body);
  });
}
//# sourceMappingURL=cli.js.map